using System;
using System.IO;
using System.Threading;
using System.Runtime.InteropServices;
//#define BUFSIZE 65540
//#define PCAP_ERRBUF_SIZE 256

namespace DotGNU.Pcap
{

//http://www.dotgnu.org/pnetlib-doc/System/Runtime/InteropServices/StructLayoutAttribute.html
[StructLayout(LayoutKind.Sequential)]
public struct PacketHeader {
	long tv_sec;  // \__ struct timeval
	long tv_usec; // /
	public int caplen; // 32b
	public int len; // 32b
}

delegate void PcapCallback(String user,PacketHeader hdr,byte[] pkt);

[StructLayout(LayoutKind.Sequential)]
public struct Stats {
    public uint ps_recv;      /* number of packets received */
    public uint ps_drop;      /* number of packets dropped */
    public uint ps_ifdrop;    /* drops by interface XXX not yet supported */
};

internal sealed class Pcap
   {
	private byte[] errbuf;
	private int _datalink=0;
	public String iface;
	public String expr;
	private IntPtr pd; /* pcap descriptor */
	private bool running=false;
	private Thread th;
	private static PcapCallback callback;

	public Pcap()
	{
	 errbuf=new byte[1024];
/*	  try {
	  iface=pcap_lookupdev(0);
	  } catch (Exception) {
		throw new NotSupportedException();
	  }*/
	}

	private void PcapLoop()
	{
		running=true;
		pcap_loop(pd,-1,callback,0);
	}
	
	public Stats getStats()
	{
	Stats st;
	if (0!=pcap_stats(pd,st))
		{
		throw new Exception("Cannot get stats from device.");
		}
	return st;
	}

	public bool Start(bool background) // Start in background?
	{
	  if (running==true)
		return false;
	  //if (callback) // TODO
	//	return false;

	  pd=pcap_open_live(iface,65535,1,69,errbuf);
	  if (pd==(IntPtr)0) 
		return false; 

	  if (background)
	  {
	  	th=new Thread( new ThreadStart(PcapLoop) );
	  	th.Start();
	  } else {
		PcapLoop();
	  }

	  return true;
	}

	public bool Stop()
	{
	if (!running)
		{ return false; }
	pcap_close(pd);
	running=false;
	return true;
	}
	public String getVersion()
	{
	String  ver=String.Format("{0}_{1}",
		pcap_major_version(),
		pcap_minor_version());
	return ver;
	}

	public void setCallback(PcapCallback func)
	{
		callback=func;
	}

	// NATIVE //
   	[DllImport("pcap")]
	extern private static IntPtr pcap_open_live(
		String iface, int a,int b, int c,byte[] errbuf);
	[DllImport("pcap")]
	extern private static String pcap_lookupdev(
		int nul);
		//String errbuf);
	[DllImport("pcap")]
	extern private static void pcap_close(IntPtr pd);
	[DllImport("pcap")]
	extern private static void pcap_loop(
		IntPtr pd,int n,PcapCallback func,int n2);
	[DllImport("pcap")]
	extern private static int pcap_stats(
		IntPtr pd,Stats st);
	[DllImport("pcap")]
	extern private static int pcap_major_version();
	[DllImport("pcap")]
	extern private static int pcap_minor_version();
   }
}
